// the local service
var ports = {};

// verbose logging flag
var verbose = false;

// send a message to the extension
function sendExtensionMessage(tab, message) {
  if (verbose)
    console.log('Sending message [' + JSON.stringify(message) + '] to tab [' + tab + ']');

  // check connection status
  if (ports[tab]) {
    message[ P_STATUS ] = S_CONNECTED;
  } else {
    message[ P_STATUS ] = S_DISCONNECTED;
  }

  // set the result status  
  if (!message[ P_RESULT ]) {
    message[ P_RESULT ] = S_OK;
  }

  chrome.tabs.sendMessage(tab, message, function(response) {
    console.log('Message sent to tab [' + tab + ']');
  });
}

// send a message to the native host
function sendNativeMessage(tab, message) {
  // check for a port
  if (!ports[tab]) {
    connect(tab);
    if (!ports[tab]) {
      // no connection
      console.log('Unable to connect to native host');
      return;
    }
  }

  if (verbose)
    console.log('Sending message [' + JSON.stringify(message) + '] for tab [' + tab + ']');

  try {
    ports[tab].postMessage(message);
  } catch (err) {
    console.log('Error sending message to native host, will try single call instance.');
    delete ports[tab];
    chrome.runtime.sendNativeMessage(NATIVE_HOST, message, function(response) {
      sendExtensionMessage(tab, response);
    });
  }

  console.log('Message sent for tab [' + tab + ']');
}

// connect to the native host
function connect(tab) {
  console.log('Connect to native host [' + NATIVE_HOST + '] for tab [' + tab + ']');

  if (!ports[tab]) {
    var port = chrome.runtime.connectNative(NATIVE_HOST);

    if (!port) {
      // unable to connect
      var response = new Object();
      response[ P_ACTION ] = A_CONNECT;
      response[ P_RESULT ] = S_ERROR;
      response[ P_REASON ] = 'Unable to connect to the native host';
      sendExtensionMessage(tab, response);
      return;
    }

    // add listeners
    port.onMessage.addListener(function(response) {
      if (verbose)
        console.log('Received message [' + JSON.stringify(response) + '] for tab [' + tab + ']');
      sendExtensionMessage(tab, response);
    });

    port.onDisconnect.addListener(function() {
      console.log('Native host disconnected for tab [' + tab + ']');
      delete ports[tab];
      var response = new Object();
      response[ P_ACTION ] = A_DISCONNECT;
      response[ P_RESULT ] = S_OK;
      sendExtensionMessage(tab, response);
    });

    // store the port
    ports[tab] = port;
  } else {
    console.log('Native host already connected for tab [' + tab + ']');
  }

  var response = new Object();
  response[ P_ACTION ] = A_CONNECT;
  response[ P_RESULT ] = S_OK;
  sendExtensionMessage(tab, response);
}

// disconnect the native host
function disconnect(tab) {
  console.log('Disconnecting native host for tab [' + tab + ']');

  if(ports[tab]) {
    ports[tab].disconnect();
  }

  var response = new Object();
  response[ P_ACTION ] = A_DISCONNECT;
  response[ P_RESULT ] = S_OK;
//  sendExtensionMessage(tab, response);
}

// listen for messages from the extension
chrome.runtime.onMessage.addListener(
  function(request, sender, sendResponse) {
    if (sender.id !== chrome.runtime.id) {
    // ignore anything not from our extension
      return;
    }

    if (request.verbose !== null)
      verbose = request.verbose === "ON" ? true : false;

    // check for control actions
    var action = request[P_ACTION];

    if (action) {
      if (action == A_CONNECT) {
        connect(sender.tab.id);
      } else if (action == A_DISCONNECT) {
        disconnect(sender.tab.id);
      } else {
        // unsupported action
        console.log('Unsupported [' + P_ACTION + '] [' + action + ']');
        var response = new Object();
        response[ P_ACTION ] = action;
        response[ P_RESULT ] = S_ERROR;
        response[ P_REASON ] = 'Unsupported action';
        sendResponse(response);
      }
    } else {
      // pass through to the native host
      sendNativeMessage(sender.tab.id, request);
    }
  }
);
